function Price = JuZhong(S,K,r,q,v,T,phi)

% Approximation for American options, following the method of Ju and Zhong,
% "An Approximate Formula for Pricing American Options".

% S = spot price
% K = strike price
% r = risk free rate
% q = dividend yield
% v = volatility
% T = maturity
% phi = 1 for call, -1 for put

% Inline function for the Black Scholes call
C = inline('s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T))',...
	's','K','r','q','v','T');

% Inline function for the Black Scholes put
P = inline('K*exp(-r*T)*normcdf(-(log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) + v*sqrt(T)) - s*exp(-q*T)*normcdf(-(log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T))',...
	's','K','r','q','v','T');

% Find the threshold S*
options = optimset('MaxFunEvals', 1e4, 'MaxIter', 1e4);
Sx = fminsearch(@(s) findS(s,K,r,q,v,T,phi), abs(S-K), options);

% European Black Scholes prices with S = S*
if phi==1
	Ve = C(Sx,K,r,q,v,T);
elseif phi==-1
	Ve = P(Sx,K,r,q,v,T);
end

% Parameters
h = (1-exp(-r*T));
alpha = 2*r/v^2;
beta = 2*(r-q)/v^2;
hAh = phi*(Sx-K) - Ve;
d1 = (log(Sx/K) + (r-q+v^2/2)*T)/v/sqrt(T);
d2 = d1 - v*sqrt(T);

% The case r=0 for calls is treated separately.
if r==0 & phi==1
	lambda = ((1-beta) + sqrt((beta-1)^2 + 8/v^2/T))/2;
	b = -2/v^4/T^2/((beta-1)^2 + 8/v^2/T);
	c = -1/sqrt((beta-1)^2 + 8/v^2/T)*...
		  (Sx*normpdf(d1)*exp(-q*T)/hAh/v/sqrt(T) ...
		 - 2*q*Sx*normcdf(d1)*exp(-q*T) ...
		 + 2/v^2/T - 4/v^4/T^2/((beta-1)^2 + 8/v^2/T));
else
	lambda  = ((1-beta) + phi*sqrt((beta-1)^2 + 4*alpha/h))/2;
	lambdap = -phi*alpha / h^2 / sqrt((beta-1)^2 + 4*alpha/h);
	dVdh = Sx*normpdf(d1)*v*exp((r-q)*T)/2/r/sqrt(T)... 
		 - phi*q*Sx*normcdf(phi*d1)*exp((r-q)*T)/r ...
	     + phi*K*normcdf(phi*d2);
	b = (1-h)*alpha*lambdap/2/(2*lambda+beta-1);
	c = -(1-h)*alpha/(2*lambda+beta-1)*(1/hAh*dVdh + 1/h + lambdap/(2*lambda+beta-1));
end

X = b*(log(S/Sx))^2 + c*log(S/Sx);

% European option prices with S = Spot
if phi == 1
	VE = C(S,K,r,q,v,T);
elseif phi == -1
	VE = P(S,K,r,q,v,T);
end

% Return the prices.
if phi*(Sx-S)>0
	Price = VE + hAh*(S/Sx)^lambda/(1-X);
else
	Price = phi*(S-K);
end
